import dataclasses

import numpy as np


@dataclasses.dataclass
class ImageData:

    class ImageSizeChangedWhileTakingBackground(Exception):
        def __init__(self):
            super().__init__("Image size changed while taking background")

    class ImageSizeChangedWhileRunning(Exception):
        def __init__(self):
            super().__init__("Image size changed while running")

    class _ImageSizeChanged(Exception):  # marker exception
        pass

    camera_address_1: str
    camera_address_2: str

    acquisition_rate: float = None

    image_single: np.ndarray = None
    image_average: np.ndarray = None

    average_samples: int = 0
    _average_image: np.ndarray = None
    to_correct_threshold: bool = False
    threshold: float = None

    to_take_background: bool = False
    to_correct_background: bool = False
    _background_sample: int = None
    background_samples: int = None
    _background_image: np.ndarray = None

    to_correct_hot_pixel: bool = False
    hot_pixel_threshold: int = None

    def initialize(self, background_samples):
        self.background_samples = background_samples
        self.to_take_background = True
        self._background_sample = 0

    def update(self, image):
        self.image_single = image.astype(np.float32)
        if self.to_take_background:
            self._apply_background_taking()
        else:
            if self.to_correct_background:
                self._apply_background_subtraction()
            if self.to_correct_threshold:
                self._apply_thresholding()
            self._apply_image_averaging()

        if self.to_correct_hot_pixel:
            self._apply_hot_pixel_correction()

    def _apply_background_taking(self):
        try:
            self._background_sample, self._background_image = self._averaging(
                self._background_sample, self._background_image, self.image_single)
        except self._ImageSizeChanged as err:
            raise self.ImageSizeChangedWhileTakingBackground() from err
        self.image_average = self._background_image.copy()

    def _apply_background_subtraction(self):
        if self.image_single.shape == self._background_image.shape:
            self.image_single -= self._background_image
        else:
            raise self.ImageSizeChangedWhileRunning()

    def _apply_thresholding(self):
        self.image_single[self.image_single < self.threshold] = 0

    def _apply_image_averaging(self):
        try:
            self.average_samples, self._average_image = self._averaging(
                self.average_samples, self._average_image, self.image_single)
        except self._ImageSizeChanged as err:
            raise self.ImageSizeChangedWhileRunning() from err
        self.image_average = self._average_image.copy()

    def _apply_hot_pixel_correction(self):
        min_value = self._background_image.min()
        max_value = self._background_image.max()
        hot_pixels = (self._background_image > min_value + self.hot_pixel_threshold/100 * (max_value - min_value))
        self.image_single[hot_pixels] = self.image_single.min()
        self.image_average[hot_pixels] = self.image_average.min()

    @classmethod
    def _averaging(cls, samples, average, image):
        samples += 1
        if samples == 1:
            average = image
        elif image.shape == average.shape:
            inverse_samples = 1 / samples
            average = (1 - inverse_samples) * average + inverse_samples * image
        else:
            raise cls._ImageSizeChanged()
        return samples, average

    def get_progress(self):
        return self._background_sample / self.background_samples * 100

    def sample_number(self):
        if self.to_take_background:
            return self._background_sample
        return self.average_samples
