from pyqtgraph import setConfigOptions, GraphicsLayoutWidget, LabelItem, ImageItem, HistogramLUTItem


class ViewerPlotter:

    setConfigOptions(imageAxisOrder="row-major")

    def __init__(self):
        self._widget = GraphicsLayoutWidget()

        self._css_begin, self._css_end = "<p><span style='white-space: pre; font-family: monospace'>", "</span></p>"
        self._label_single = LabelItem(justify="center")
        self._widget.addItem(self._label_single)

        self._widget.addItem(LabelItem())

        self._label_average = LabelItem(justify="center")
        self._widget.addItem(self._label_average)

        self._widget.nextRow()

        self._image_single = ImageItem()
        self._image_single_plot = self._widget.addPlot()
        self._image_single_plot.addItem(self._image_single)
        self._hist_single = HistogramLUTItem()
        self._hist_single.setImageItem(self._image_single)
        self._widget.addItem(self._hist_single)

        self._image_average = ImageItem()
        self._image_average_plot = self._widget.addPlot()
        self._image_average_plot.addItem(self._image_average)
        self._hist_average = HistogramLUTItem()
        self._hist_average.setImageItem(self._image_average)
        self._widget.addItem(self._hist_average)
        self._initialize_plots()
        self._auto_scaling_set_once = False

    def _initialize_plots(self):
        self._initialize_labels()
        self._initialize_image_plot()

    def _initialize_labels(self):
        self.set_acquisition_rate(0.0)
        self.set_average_samples(0)

    def _initialize_image_plot(self):
        self._image_single_plot.setMenuEnabled(False)
        self._image_single_plot.showAxis("top", show=True)
        self._image_single_plot.showAxis("right", show=True)
        self._image_single_plot.getAxis("top").setStyle(showValues=False, tickLength=0)
        self._image_single_plot.getAxis("right").setStyle(showValues=False, tickLength=0)
        self._image_single_plot.getAxis("left").setStyle(showValues=False, tickLength=0)
        self._image_single_plot.getAxis("bottom").setStyle(showValues=False, tickLength=0)
        self._image_single_plot.invertY(True)
        self._image_single_plot.getViewBox().setAspectLocked()
        self._image_average_plot.setMenuEnabled(False)
        self._image_average_plot.showAxis("top", show=True)
        self._image_average_plot.showAxis("right", show=True)
        self._image_average_plot.getAxis("top").setStyle(showValues=False, tickLength=0)
        self._image_average_plot.getAxis("right").setStyle(showValues=False, tickLength=0)
        self._image_average_plot.getAxis("left").setStyle(showValues=False, tickLength=0)
        self._image_average_plot.getAxis("bottom").setStyle(showValues=False, tickLength=0)
        self._image_average_plot.invertY(True)
        self._image_average_plot.getViewBox().setAspectLocked()

    @property
    def widget(self):
        return self._widget

    def set_acquisition_rate(self, rate):
        self._label_single.setText(self._css_begin + f"acquisition rate: {rate:2.0f} Hz" + self._css_end)

    def set_average_samples(self, samples):
        self._label_average.setText(self._css_begin + f"average samples: {samples}" + self._css_end)

    def set_images(self, image_single, image_average):
        self._image_single.setImage(image_single)
        self._image_average.setImage(image_average)
        if not self._auto_scaling_set_once:
            self._hist_single.setLevels(image_single.min(), image_single.max())
            self._hist_average.setLevels(image_average.min(), image_average.max())
            self._auto_scaling_set_once = True
        self._set_levels()

    def _set_levels(self):
        self._hist_single.region.lineMoved()
        self._hist_single.region.lineMoveFinished()
        self._hist_average.region.lineMoved()
        self._hist_average.region.lineMoveFinished()

    def reset(self):
        self._auto_scaling_set_once= False

    def clear(self):
        self._image_single.clear()
        self._hist_single.plot.setData([], [])
        self._hist_single.region.setRegion([0, 1])
        self._image_average.clear()
        self._hist_average.plot.setData([], [])
        self._hist_average.region.setRegion([0, 1])
        self.set_average_samples(0)
