import time

import pydoocs
#from . import mydoocs as pydoocs


class IOService:

    class IOException(Exception):
        def __init__(self, err):
            super().__init__("IO error: " + str(err))

    @classmethod
    def _read_from(cls, address):
        try:
            return pydoocs.read(address)
        except Exception as err:
            raise cls.IOException(err)

    @classmethod
    def _write_to(cls, address, data):
        try:
            pydoocs.write(address, data)
        except Exception as err:
            raise cls.IOException(err)

    def __init__(self, image_data):
        self._image_data = image_data
        self._cameras = {"CMOS1": self._image_data.camera_address_1, "CMOS2": self._image_data.camera_address_2}
        self._camera_address = None
        self._jpeg_compression_state_address = None
        self.use_camera("CMOS1")
        self._is_new_data_available = None
        self._event = None
        self._last_update = None
        self._image_dict = None

    @property
    def image_data(self):
        return self._image_data

    @property
    def cameras(self):
        return self._cameras

    @property
    def is_new_data_available(self):
        return self._is_new_data_available

    def initialize(self):
        self._event = self._get_event()
        self._last_update = time.time()

    def _get_event(self):
        try:
            self._image_dict = self._read_from(self._camera_address)
            return self._image_dict["miscellaneous"]["event"]
        except KeyError:
            raise self.IOException("Camera does not generate events")

    def update(self):
        self._acquire_data()

    def _acquire_data(self):
        self._is_new_data_available = False
        event = self._get_event()
        if event != self._event:
            self._update_acquisition_rate()
            self._update_image_data()
            self._is_new_data_available = True
            self._event = event

    def _update_acquisition_rate(self):
        current_update = time.time()
        self._image_data.acquisition_rate = 1 / (current_update - self._last_update)
        self._last_update = current_update

    def _update_image_data(self):
        self._image_data.update(self._image_dict["data"])

    def use_camera(self, camera):
        self._camera_address = self._cameras[camera]
        self._jpeg_compression_state_address = "/".join(self._camera_address.split("/")[:-1] + ["COMP_MODE"])

    def get_jpeg_compression_state(self):
        return self._read_from(self._jpeg_compression_state_address)["data"] >= 0

    def use_jpeg_compression(self, to_use_jpeg_compression):
        if to_use_jpeg_compression:
            self._write_to(self._jpeg_compression_state_address, 0)
        else:
            self._write_to(self._jpeg_compression_state_address, -1)

    def use_zmq_extension(self, to_use_zmq_extension):
        extension = "_ZMQ"
        if to_use_zmq_extension:
            self._camera_address = self._camera_address + extension
        else:
            self._camera_address = self._camera_address[:-len(extension)]
