import sys
import traceback

from .camposkop_tool_ui import CamposkopToolUi
from .print_tool import PrintTool
from .viewer_tool import ViewerTool


class CamposkopTool:
    APPLICATION_NAME = CamposkopToolUi.APPLICATION_NAME
    VERSION = CamposkopToolUi.VERSION

    def __init__(self, e_log_service, concurrency_service, io_service):
        self._print_tool = PrintTool(e_log_service.e_log_data, self.APPLICATION_NAME, self.VERSION)
        self._viewer_tool = ViewerTool(io_service.image_data)
        self._camposkop_tool_ui = CamposkopToolUi(self._viewer_tool.ui, cameras=io_service.cameras)
        self._e_log_service = e_log_service
        self._concurrency_service = concurrency_service
        self._io_service = io_service
        self._initialize_tools_with_ui_settings()
        self._create_connections()

    def _initialize_tools_with_ui_settings(self):
        self._update_rate_options()

    def _create_connections(self):
        self._create_camposkop_tool_ui_connections()
        self._create_viewer_tool_connections()

    def _create_camposkop_tool_ui_connections(self):
        self._camposkop_tool_ui.print_to_e_log_action.triggered.connect(self._print_to_e_log)
        self._camposkop_tool_ui.camera_menu.triggered.connect(self._update_camera)
        self._checked_camera = self._camposkop_tool_ui.get_checked_camera()
        self._camposkop_tool_ui.change_viewer_tab_text(self._checked_camera)
        self._camposkop_tool_ui.image_menu.aboutToShow.connect(self._update_jpeg_compression_state)
        self._camposkop_tool_ui.apply_jpeg_compression.triggered.connect(self._apply_jpeg_compression)
        self._camposkop_tool_ui.apply_zmq_extension.triggered.connect(self._apply_zmq_extension)
        self._camposkop_tool_ui.update_menu.triggered.connect(self._update_rate_options)

    def _print_to_e_log(self):
        self._print_to_e_log_with(self._print_tool.show_standard_dialog)

    def _update_camera(self):
        self._io_service.use_camera(self._camposkop_tool_ui.get_checked_camera())
        if self._camposkop_tool_ui.get_checked_camera() != self._checked_camera:
            self._checked_camera = self._camposkop_tool_ui.get_checked_camera()
            self._camposkop_tool_ui.change_viewer_tab_text(self._checked_camera)
            self._viewer_tool.set_apply_background_correction_check_box_to(False)
            self._viewer_tool.set_apply_hot_pixel_correction_check_box_to(False)
            self._viewer_tool.reset_button_clicked()
            self._viewer_tool.ui.clear_plot()

    def _print_to_e_log_with(self, show_dialog):
        self._e_log_service.update_e_log_data_image(self._camposkop_tool_ui.screenshot())
        show_dialog()
        self._e_log_service.print()

    def _update_jpeg_compression_state(self):
        try:
            self._camposkop_tool_ui.apply_jpeg_compression.setChecked(self._io_service.get_jpeg_compression_state())
            self._camposkop_tool_ui.apply_jpeg_compression.setEnabled(True)
        except self._io_service.IOException:
            self._camposkop_tool_ui.show_error_with("Can't get jpeg compression state")
            self._camposkop_tool_ui.apply_jpeg_compression.setEnabled(False)

    def _apply_jpeg_compression(self):
        try:
            self._io_service.use_jpeg_compression(self._camposkop_tool_ui.apply_jpeg_compression.isChecked())
            if not self._camposkop_tool_ui.apply_zmq_extension.isChecked():
                self._viewer_tool.reset_button_clicked()
        except self._io_service.IOException:
            self._camposkop_tool_ui.show_error_with("Can't change jpeg compression state")

    def _apply_zmq_extension(self):
        self._io_service.use_zmq_extension(self._camposkop_tool_ui.apply_zmq_extension.isChecked())
        if self._camposkop_tool_ui.apply_jpeg_compression.isChecked():
            self._viewer_tool.reset_button_clicked()

    def _update_rate_options(self):
        self._viewer_tool.use_checked_rate(self._camposkop_tool_ui.get_checked_update_option())

    def _create_viewer_tool_connections(self):
        self._viewer_tool.ui.start_button.clicked.connect(self._start_io_and_plot_loop)
        self._viewer_tool.ui.stop_button.clicked.connect(self._stop_io_and_plot_loop)
        self._viewer_tool.ui.take_button.clicked.connect(self._start_background_taking_loop)

    def _start_io_and_plot_loop(self):
        try:
            self._io_service.initialize()
            self._concurrency_service.start(self._io_and_plot_loop)
        except self._io_service.IOException as err:
            self._viewer_tool.ui.stop_button.click()
            self._camposkop_tool_ui.show_error_with(str(err))

    def _io_and_plot_loop(self):
        try:
            self._io_service.update()
            if self._io_service.is_new_data_available:
                self._viewer_tool.update()
        except (self._io_service.IOException, self._io_service.image_data.ImageSizeChangedWhileTakingBackground) as err:
            self._viewer_tool.ui.stop_button.click()
            self._viewer_tool.ui.clear_plot()
            self._camposkop_tool_ui.show_error_with(str(err))
        except self._io_service.image_data.ImageSizeChangedWhileRunning as err:
            self._viewer_tool.ui.stop_button.click()
            self._viewer_tool.reset_button_clicked()
            self._viewer_tool.set_apply_background_correction_check_box_to(False)
            self._viewer_tool.set_apply_hot_pixel_correction_check_box_to(False)
            self._camposkop_tool_ui.show_error_with(str(err))

    def _stop_io_and_plot_loop(self):
        self._concurrency_service.stop()

    def _start_background_taking_loop(self):
        if self._viewer_tool.is_started:
            self._start_io_and_plot_loop()

    def show_window(self):
        self._camposkop_tool_ui.show_window()

    def exception_handler(self, exception_type, exception_value, exception_traceback):
        self._concurrency_service.stop()
        self._camposkop_tool_ui.close()  # gives extra visual hint for erroneous situation
        exception = exception_type.__name__ + ": unexpected exception occurred."
        message = "Traceback (most recent call last):\n"
        message += "".join(traceback.format_list(traceback.extract_tb(exception_traceback))) + "\n"
        message += exception_type.__name__ + ": " + str(exception_value)
        sys.exit(self._print_to_e_log_with(lambda: self._print_tool.show_error_dialog(exception, message)))
