from PyQt5.QtCore import QByteArray, QBuffer, QIODevice
from PyQt5.QtWidgets import QMainWindow, QWidget, QVBoxLayout, QAction, QTabWidget, QMessageBox, QActionGroup


class CamposkopToolUi(QMainWindow):

    APPLICATION_NAME = "Camposkop"
    VERSION = "0.1.5"

    def __init__(self, *tool_uis, cameras):
        super().__init__()
        self._tool_uis = tool_uis
        self._cameras_items = cameras
        self._cameras = list(self._cameras_items.keys())
        self.setWindowTitle(self.APPLICATION_NAME)
        self._main_layout = QVBoxLayout()
        self._print_to_e_log_action = QAction("Print to E-Log")
        self._camera_menu = None
        self._image_menu = None
        self._apply_jpeg_compression = QAction("jpeg compression", checkable=True)
        self._apply_zmq_extension = QAction("_ZMQ extension", checkable=True)
        self._update_menu = None
        self._update_items = {"x1/1": 1, "x1/2": 2, "x1/4": 4, "x1/8": 8, "x1/16": 16}
        self._update_menu_items = list(self._update_items.keys())
        self._click_about_action = QAction("About")
        self._tab_widget = QTabWidget()
        self._create_layout()
        self._create_connections()

    def _create_layout(self):
        self._create_main_layout()
        self._create_menu_bar()
        self._create_tools_tab_widget()

    def _create_main_layout(self):
        main_widget = QWidget()
        main_widget.setLayout(self._main_layout)
        self.setCentralWidget(main_widget)

    def _create_menu_bar(self):
        self._create_file_menu()
        self._create_camera_menu()
        self._create_image_menu()
        self._create_update_menu()
        self._create_help_menu()

    def _create_file_menu(self):
        self.menuBar().addMenu("File").addAction(self._print_to_e_log_action)

    def _create_camera_menu(self):
        self._camera_menu = self.menuBar().addMenu("Camera")
        camera_menu_action_group = QActionGroup(self._camera_menu)
        for index, camera in enumerate(self._cameras):
            self._cameras[index] = QAction(camera, checkable=True), camera
            self._camera_menu.addAction(camera_menu_action_group.addAction(self._cameras[index][0]))
        self._cameras[0][0].setChecked(True)

    def _create_image_menu(self):
        self._image_menu = self.menuBar().addMenu("Image")
        self._image_menu.addAction(self._apply_jpeg_compression)
        self._image_menu.addAction(self._apply_zmq_extension)

    def _create_update_menu(self):
        self._update_menu = self.menuBar().addMenu("Update")
        update_menu_action_group = QActionGroup(self._update_menu)
        for index, menu_item in enumerate(self._update_menu_items):
            self._update_menu_items[index] = QAction(menu_item, checkable=True), menu_item
            self._update_menu.addAction(update_menu_action_group.addAction(self._update_menu_items[index][0]))
        self._update_menu_items[0][0].setChecked(True)

    def _create_help_menu(self):
        self.menuBar().addMenu("Help").addAction(self._click_about_action)

    def _create_tools_tab_widget(self):
        for tool_ui in self._tool_uis:
            self._tab_widget.addTab(tool_ui, tool_ui.NAME)
        self._main_layout.addWidget(self._tab_widget)

    def _create_connections(self):
        self._click_about_action.triggered.connect(self._show_about)

    @property
    def print_to_e_log_action(self):
        return self._print_to_e_log_action

    @property
    def camera_menu(self):
        return self._camera_menu

    @property
    def image_menu(self):
        return self._image_menu

    @property
    def apply_jpeg_compression(self):
        return self._apply_jpeg_compression

    @property
    def apply_zmq_extension(self):
        return self._apply_zmq_extension

    @property
    def update_menu(self):
        return self._update_menu

    @property
    def tab_widget(self):
        return self._tab_widget

    def get_checked_camera(self):
        for camera in self._cameras:
            if camera[0].isChecked():
                return camera[1]
        return None

    def get_checked_update_option(self):
        for menu_item in self._update_menu_items:
            if menu_item[0].isChecked():
                return self._update_items[menu_item[1]]
        return None

    def _show_about(self):
        QMessageBox.about(self, self.APPLICATION_NAME, "Version " + self.VERSION)

    def change_viewer_tab_text(self, camera):
        self._tab_widget.setTabText(0, self._tool_uis[0].NAME + camera)

    def screenshot(self):
        screenshot_tmp = QByteArray()
        screenshot_buffer = QBuffer(screenshot_tmp)
        screenshot_buffer.open(QIODevice.WriteOnly)
        widget = QWidget.grab(self)
        widget.save(screenshot_buffer, "png")
        return screenshot_tmp.toBase64().data().decode()

    @staticmethod
    def show_error_with(message):
        message_box = QMessageBox(icon=QMessageBox.Critical)
        message_box.setText(message)
        message_box.exec()

    def show_window(self):
        self.show()
        self.setMinimumSize(self.width(), self.height())
