from .viewer_tool_ui import ViewerToolUi


class ViewerTool:

    def __init__(self, image_data):
        self._viewer_tool_ui = ViewerToolUi()
        self._image_data = image_data
        self._update_multiples = None
        self._is_started = False
        self._create_connections()

    def _create_connections(self):
        self._viewer_tool_ui.start_button.clicked.connect(self._start_button_clicked)
        self._viewer_tool_ui.stop_button.clicked.connect(self._stop_button_clicked)
        self._viewer_tool_ui.reset_button.clicked.connect(self.reset_button_clicked)
        self._viewer_tool_ui.take_button.clicked.connect(self._take_button_clicked)
        self._viewer_tool_ui.enable_threshold_check_box.toggled.connect(self._set_averaging_threshold)
        self._viewer_tool_ui.apply_background_correction_check_box.stateChanged.connect(self._set_background_correction)
        self._viewer_tool_ui.hot_pixel_threshold_slider.valueChanged.connect(self._set_hot_pixel_threshold)
        self._viewer_tool_ui.apply_hot_pixel_correction_check_box.stateChanged.connect(self._set_hot_pixel_correction)

    @property
    def ui(self):
        return self._viewer_tool_ui

    @property
    def is_started(self):
        return self._is_started

    def use_checked_rate(self, update_multiples):
        self._update_multiples = update_multiples

    def _start_button_clicked(self):
        self._viewer_tool_ui.start_button.setEnabled(False)
        self._viewer_tool_ui.take_button.setEnabled(False)
        self._viewer_tool_ui.start_button.setText("running ...")
        self._is_started = True

    def _stop_button_clicked(self):
        if self._is_started:
            self._is_started = False
            self._viewer_tool_ui.start_button.setText(ViewerToolUi.START)
            self._viewer_tool_ui.take_button.setEnabled(True)
            self._viewer_tool_ui.show_take_button()
            self._viewer_tool_ui.start_button.setEnabled(True)
            self._viewer_tool_ui.samples_line_edit.setEnabled(True)
            self._viewer_tool_ui.enable_threshold_check_box.setEnabled(True)
            if not self._viewer_tool_ui.enable_threshold_check_box.isChecked():
                self._viewer_tool_ui.threshold_line_edit.setEnabled(True)
            self._viewer_tool_ui.reset_button.setEnabled(True)
            self._image_data.to_take_background = False
            self._viewer_tool_ui.update_acquisition_rate(0.0)
            self._viewer_tool_ui.reset_plot()

    def reset_button_clicked(self):
        self._image_data.average_samples = 0
        self._viewer_tool_ui.update_average_samples(self._image_data.average_samples)
        self._viewer_tool_ui.clear_plot()

    def _take_button_clicked(self):
        if self._is_background_input_validation_successful():
            self._viewer_tool_ui.start_button.setEnabled(False)
            self._viewer_tool_ui.reset_button.setEnabled(False)
            self._viewer_tool_ui.threshold_line_edit.setEnabled(False)
            self._viewer_tool_ui.enable_threshold_check_box.setEnabled(False)
            self._viewer_tool_ui.samples_line_edit.setEnabled(False)
            self._viewer_tool_ui.show_progress_bar()
            self._image_data.initialize(int(self._viewer_tool_ui.samples_line_edit.text()))
            self.set_apply_background_correction_check_box_to(False)
            self.set_apply_hot_pixel_correction_check_box_to(False)
            self._is_started = True

    def _is_background_input_validation_successful(self):
        if not self._viewer_tool_ui.samples_line_edit.text():
            self._viewer_tool_ui.show_error_with("Empty background correction 'Samples' field")
            return False
        return True

    def set_apply_background_correction_check_box_to(self, state):
        self._viewer_tool_ui.apply_background_correction_check_box.setChecked(state)
        self._viewer_tool_ui.apply_background_correction_check_box.setEnabled(state)

    def set_apply_hot_pixel_correction_check_box_to(self, state):
        self._viewer_tool_ui.apply_hot_pixel_correction_check_box.setChecked(state)
        self._viewer_tool_ui.apply_hot_pixel_correction_check_box.setEnabled(state)

    def _set_averaging_threshold(self):
        self.reset_button_clicked()
        if self._viewer_tool_ui.enable_threshold_check_box.isChecked():
            if (not self._viewer_tool_ui.threshold_line_edit.text() or
                    self._viewer_tool_ui.threshold_line_edit.text() == "."):
                self._viewer_tool_ui.stop_button.click()
                self._viewer_tool_ui.show_error_with("Empty/Faulty averaging 'Threshold' field")
                self._viewer_tool_ui.enable_threshold_check_box.setChecked(False)
                return
            self._viewer_tool_ui.threshold_line_edit.setEnabled(False)
            self._image_data.to_correct_threshold = True
            self._image_data.threshold = float(self._viewer_tool_ui.threshold_line_edit.text())
        else:
            self._viewer_tool_ui.threshold_line_edit.setEnabled(True)
            self._image_data.to_correct_threshold = False

    def _set_background_correction(self):
        self._image_data.to_correct_background = self._viewer_tool_ui.apply_background_correction_check_box.isChecked()
        self._image_data.average_samples = 0
        self._viewer_tool_ui.reset_plot()

    def _set_hot_pixel_threshold(self):
        self._image_data.hot_pixel_threshold = self._viewer_tool_ui.hot_pixel_threshold_slider.value()

    def _set_hot_pixel_correction(self):
        self._image_data.to_correct_hot_pixel = self._viewer_tool_ui.apply_hot_pixel_correction_check_box.isChecked()
        self._image_data.hot_pixel_threshold = self._viewer_tool_ui.hot_pixel_threshold_slider.value()

    def update(self):
        self._update_plots()
        self._update_labels()
        self._update_background_taking_progress()

    def _update_plots(self):
        if self._to_update():
            self._viewer_tool_ui.plot_images(self._image_data.image_single,
                                             self._image_data.image_average)

    def _to_update(self):
        return (self._image_data.sample_number() - 1) % self._update_multiples == 0

    def _update_labels(self):
        self._viewer_tool_ui.update_acquisition_rate(self._image_data.acquisition_rate)
        self._viewer_tool_ui.update_average_samples(self._image_data.average_samples)

    def _update_background_taking_progress(self):
        if self._image_data.to_take_background:
            self._viewer_tool_ui.set_progress(self._image_data.get_progress())
            if self._image_data.get_progress() == 100:
                self.set_apply_background_correction_check_box_to(True)
                self.set_apply_hot_pixel_correction_check_box_to(True)
                self._viewer_tool_ui.stop_button.click()
