from PyQt5.QtCore import Qt, QRegExp
from PyQt5.QtGui import QRegExpValidator
from PyQt5.QtWidgets import (QMainWindow, QWidget, QVBoxLayout, QHBoxLayout, QGroupBox, QPushButton, QLineEdit, QLabel,
                             QSizePolicy, QProgressBar, QCheckBox, QStackedWidget, QMessageBox, QSlider)

from ..plotter.viewer_plotter import ViewerPlotter


class ViewerToolUi(QMainWindow):

    NAME = "Viewer: "

    START = "Start"

    def __init__(self):
        super().__init__()
        self._main_layout = QVBoxLayout()
        self._viewer_plotter = ViewerPlotter()
        self._start_button = self._get_push_button_with(self.START)
        self._stop_button = self._get_push_button_with("Stop")
        self._reset_button = self._get_push_button_with("Reset")
        self._threshold_line_edit = self._get_threshold_line_edit()
        self._enable_threshold_check_box = QCheckBox("Enable")
        self._take_button_and_progress_bar_stack = QStackedWidget()
        self._take_button = self._get_push_button_with("Take")
        self._progress_bar = QProgressBar()
        self._samples_line_edit = self._get_samples_line_edit()
        self._apply_background_correction_check_box = QCheckBox("Apply", enabled=False)
        self._hot_pixel_threshold_slider = self._get_hot_pixel_threshold_slider()
        self._apply_hot_pixel_correction_check_box = QCheckBox("Apply", enabled=False)
        self._create_layout()

    @staticmethod
    def _get_push_button_with(label):
        push_button = QPushButton(label)
        push_button.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Preferred)
        return push_button

    @staticmethod
    def _get_threshold_line_edit():
        threshold_line_edit = QLineEdit("0.0")
        threshold_line_edit.setAlignment(Qt.AlignRight)
        threshold_line_edit.setMaxLength(5)
        threshold_line_edit.setMaximumWidth(50)
        threshold_line_edit.setValidator(QRegExpValidator(QRegExp("[0-9]*[.][0-9]*")))
        return threshold_line_edit

    @staticmethod
    def _get_samples_line_edit():
        samples_line_edit = QLineEdit("10")
        samples_line_edit.setAlignment(Qt.AlignRight)
        samples_line_edit.setMaxLength(5)
        samples_line_edit.setMaximumWidth(50)
        samples_line_edit.setValidator(QRegExpValidator(QRegExp("[1-9][0-9]*")))
        return samples_line_edit

    @staticmethod
    def _get_hot_pixel_threshold_slider():
        slider = QSlider(Qt.Horizontal)
        slider.setRange(0, 100)
        slider.setValue(100)
        return slider

    def _create_layout(self):
        self._create_main_layout()
        self._create_plot_layout()
        self._create_controls_layout()

    def _create_main_layout(self):
        main_widget = QWidget()
        main_widget.setLayout(self._main_layout)
        self.setCentralWidget(main_widget)

    def _create_plot_layout(self):
        plot_layout = QHBoxLayout()
        plot_layout.addWidget(self._viewer_plotter.widget)
        self._main_layout.addLayout(plot_layout)

    def _create_controls_layout(self):
        controls_layout = QHBoxLayout()
        controls_layout.addWidget(self._create_control_group_box())
        controls_layout.addWidget(self._create_averaging_group_box())
        controls_layout.addWidget(self._create_background_correction_group_box())
        controls_layout.addWidget(self._create_hot_pixel_correction_group_box())
        self._main_layout.addLayout(controls_layout)

    def _create_control_group_box(self):
        group_box = QGroupBox("Control")
        widgets_layout = QHBoxLayout()
        widgets_layout.addWidget(self._start_button)
        widgets_layout.addWidget(self._stop_button)
        group_box.setLayout(widgets_layout)
        return group_box

    def _create_averaging_group_box(self):
        group_box = QGroupBox("Averaging")
        widgets_layout = QHBoxLayout()
        widgets_layout.addWidget(self._reset_button)
        widgets_layout.addWidget(QLabel("Threshold:"))
        widgets_layout.addWidget(self._threshold_line_edit)
        widgets_layout.addWidget(self._enable_threshold_check_box)
        group_box.setLayout(widgets_layout)
        return group_box

    def _create_background_correction_group_box(self):
        group_box = QGroupBox("Background Correction")
        group_box.setSizePolicy(QSizePolicy.Preferred, QSizePolicy.Fixed)
        widgets_layout = QHBoxLayout()
        self._take_button_and_progress_bar_stack.addWidget(self._take_button)
        self._take_button_and_progress_bar_stack.addWidget(self._progress_bar)
        widgets_layout.addWidget(self._take_button_and_progress_bar_stack)
        widgets_layout.addWidget(QLabel("Samples:"))
        widgets_layout.addWidget(self._samples_line_edit)
        widgets_layout.addWidget(self._apply_background_correction_check_box)
        group_box.setLayout(widgets_layout)
        return group_box

    def _create_hot_pixel_correction_group_box(self):
        group_box = QGroupBox("Hot Pixel Correction")
        group_box.setSizePolicy(QSizePolicy.Preferred, QSizePolicy.Fixed)
        widgets_layout = QHBoxLayout()

        widgets_layout.addWidget(self._hot_pixel_threshold_slider)
        widgets_layout.addWidget(self._apply_hot_pixel_correction_check_box)
        group_box.setLayout(widgets_layout)
        return group_box

    @property
    def start_button(self):
        return self._start_button

    @property
    def stop_button(self):
        return self._stop_button

    @property
    def reset_button(self):
        return self._reset_button

    @property
    def threshold_line_edit(self):
        return self._threshold_line_edit

    @property
    def enable_threshold_check_box(self):
        return self._enable_threshold_check_box

    @property
    def take_button(self):
        return self._take_button

    @property
    def progress_bar(self):
        return self._progress_bar

    @property
    def samples_line_edit(self):
        return self._samples_line_edit

    @property
    def apply_background_correction_check_box(self):
        return self._apply_background_correction_check_box

    @property
    def hot_pixel_threshold_slider(self):
        return self._hot_pixel_threshold_slider

    @property
    def apply_hot_pixel_correction_check_box(self):
        return self._apply_hot_pixel_correction_check_box

    @staticmethod
    def show_error_with(message):
        message_box = QMessageBox(icon=QMessageBox.Critical)
        message_box.setText(message)
        message_box.exec()

    def show_progress_bar(self):
        self._progress_bar.setValue(0)
        self._take_button_and_progress_bar_stack.setCurrentWidget(self._progress_bar)

    def show_take_button(self):
        self._take_button_and_progress_bar_stack.setCurrentWidget(self._take_button)

    def update_acquisition_rate(self, rate):
        self._viewer_plotter.set_acquisition_rate(rate)

    def update_average_samples(self, samples):
        self._viewer_plotter.set_average_samples(samples)

    def set_progress(self, value):
        self._progress_bar.setValue(value)

    def plot_images(self, image_single, image_average):
        self._viewer_plotter.set_images(image_single, image_average)

    def reset_plot(self):
        self._viewer_plotter.reset()

    def clear_plot(self):
        self._viewer_plotter.clear()
        self._viewer_plotter.reset()
